@@ -1,4 +1,8 @@
-geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussian, corstr = "independence", Mv = 1, weights = NULL, corr.mat = NULL, init.beta=NULL, init.alpha=NULL, init.phi = 1, scale.fix=FALSE, nodummy=FALSE, sandwich=TRUE, maxit=20, tol=0.00001){
+geem <- function(formula, id, waves=NULL, data = parent.frame(),
+                 family = gaussian, corstr = "independence", Mv = 1,
+                 weights = NULL, corr.mat = NULL, init.beta=NULL,
+                 init.alpha=NULL, init.phi = 1, scale.fix = FALSE, nodummy = FALSE,
+                 sandwich = TRUE, useP = TRUE, maxit = 20, tol = 0.00001){
   call <- match.call()
 
   famret <- getfam(family)
@@ -19,6 +23,8 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     stop("If scale.fix=TRUE, then init.phi must be supplied")
   }
 
+  useP <- as.numeric(useP)
+  
   ### First, get all the relevant elements from the arguments
   dat <- model.frame(formula, data, na.action=na.pass)
   nn <- dim(dat)[1]
@@ -66,6 +72,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   dat$id <- id
   dat$weights <- weights
   dat$waves <- waves
+  
 
   if(!is.numeric(dat$waves) & !is.null(dat$waves)) stop("waves must be either an integer vector or NULL")
 
@@ -158,7 +165,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   nn <- dim(dat)[1]
   K <- length(unique(id))
 
-
+  
   modterms <- terms(formula)
 
   X <- model.matrix(formula,dat)
@@ -176,11 +183,13 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     off <- offset
   }
 
+  
   # Is there an intercept column?
   interceptcol <- apply(X==1, 2, all)
 
   ## Basic check to see if link and variance functions make any kind of sense
-  linkOfMean <- LinkFun(mean(Y))
+  linkOfMean <- LinkFun(mean(Y[includedvec])) - mean(off)
+  
   if( any(is.infinite(linkOfMean) | is.nan(linkOfMean)) ){
     stop("Infinite or NaN in the link of the mean of responses.  Make sure link function makes sense for these data.")
   }
@@ -208,7 +217,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   idspl <-ifelse(tmpwgt==0, NA, id)
   includedlen <- as.numeric(summary(split(Y, idspl, drop=T))[,1])
   len <- as.numeric(summary(split(Y, id, drop=T))[,1])
-
+  
   W <- Diagonal(x=weights)
   sqrtW <- sqrt(W)
   included <- Diagonal(x=(as.numeric(weights>0)))
@@ -216,8 +225,6 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   # Get vector of cluster sizes... remember this len variable
   #len <- as.numeric(summary(split(Y, id, drop=T))[,1])
 
-
-
   # Set the initial alpha value
   if(is.null(init.alpha)){
     alpha.new <- 0.2
@@ -284,8 +291,13 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     n.vec <- vector("numeric", nn)
     index <- c(cumsum(len) - len, nn)
     for(i in 1:K){
-      n.vec[(index[i]+1) : index[i+1]] <-  rep(len[i], len[i])
+      n.vec[(index[i]+1) : index[i+1]] <-  rep(includedlen[i], len[i])
     }
+    #n.vec <- vector("numeric", nn)
+    #index <- c(cumsum(len) - len, nn)
+    #for(i in 1:K){
+    #  n.vec[(index[i]+1) : index[i+1]] <-  rep(len[i], len[i])
+    #}
   }else if(cor.match == 4){
     # M-DEPENDENT, check that M is not too large
     if(Mv >= max(len)){
@@ -357,41 +369,39 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
 
     diag(StdErr) <- sqrt(1/VarFun(mu))
 
-    ###### REMOVE LATER
-    n.vec <- vector("numeric", nn)
-    index <- c(cumsum(len) - len, nn)
-    for(i in 1:K){
-      n.vec[(index[i]+1) : index[i+1]] <-  rep(len[i], len[i])
-    }
-    R.alpha.inv <- getAlphaInvEX(0.2, n.vec, BlockDiag)
-    beta.list <- updateBeta(Y, X, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, sqrtW)
-    beta <- beta.list$beta
-    eta <- as.vector(X %*% beta) + off
-    mu <- InvLink(eta)
-    diag(StdErr) <- sqrt(1/VarFun(mu))
-    ###########
     if(!scale.fix){
-      phi <- updatePhi(Y, mu, VarFun, p, StdErr, included, includedlen, sqrtW)
+      phi <- updatePhi(Y, mu, VarFun, p, StdErr, included, includedlen, sqrtW, useP)
     }
     phi.new <- phi
-
-
+    
 
     ## Calculate alpha, R(alpha)^(-1) / phi
     if(cor.match == 2){
       # AR-1
-      alpha.new <- updateAlphaAR(Y, mu, VarFun, phi, id, len, StdErr, p, included, includedlen, includedvec, allobs, sqrtW, BlockDiag)
+      alpha.new <- updateAlphaAR(Y, mu, VarFun, phi, id, len,
+                                 StdErr, p, included, includedlen,
+                                 includedvec, allobs, sqrtW,
+                                 BlockDiag, useP)
       R.alpha.inv <- getAlphaInvAR(alpha.new, a1,a2,a3,a4, row.vec, col.vec)/phi
     }else if(cor.match == 3){
       #EXCHANGEABLE
-      alpha.new <- updateAlphaEX(Y, mu, VarFun, phi, id, len, StdErr, Resid, p, BlockDiag, included, includedlen, sqrtW)
+      alpha.new <- updateAlphaEX(Y, mu, VarFun, phi, id, len, StdErr,
+                                 Resid, p, BlockDiag, included,
+                                 includedlen, sqrtW, useP)
+      #R.alpha.inv <- getAlphaInvEX(alpha.new, n.vec, BlockDiag)/phi
       R.alpha.inv <- getAlphaInvEX(alpha.new, n.vec, BlockDiag)/phi
     }else if(cor.match == 4){
       # M-DEPENDENT
       if(Mv==1){
-        alpha.new <- updateAlphaAR(Y, mu, VarFun, phi, id, len, StdErr, p, included, includedlen, includedvec, allobs, sqrtW, BlockDiag)
+        alpha.new <- updateAlphaAR(Y, mu, VarFun, phi, id, len,
+                                   StdErr, p, included, includedlen,
+                                   includedvec, allobs,
+                                   sqrtW, BlockDiag, useP)
       }else{
-        alpha.new <- updateAlphaMDEP(Y, mu, VarFun, phi, id, len, StdErr, Resid, p, BlockDiag, Mv, included, includedlen, allobs, sqrtW)
+        alpha.new <- updateAlphaMDEP(Y, mu, VarFun, phi, id, len,
+                                     StdErr, Resid, p, BlockDiag, Mv,
+                                     included, includedlen,
+                                     allobs, sqrtW, useP)
         if(sum(len>Mv) <= p){
           unstable <- T
         }
@@ -403,7 +413,10 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
       R.alpha.inv <- getAlphaInvMDEP(alpha.new, len, row.vec, col.vec)/phi
     }else if(cor.match == 5){
       # UNSTRUCTURED
-      alpha.new <- updateAlphaUnstruc(Y, mu, VarFun, phi, id, len, StdErr, Resid,  p, BlockDiag, included, includedlen, allobs, sqrtW)
+      alpha.new <- updateAlphaUnstruc(Y, mu, VarFun, phi, id, len,
+                                      StdErr, Resid,  p, BlockDiag,
+                                      included, includedlen, allobs,
+                                      sqrtW, useP)
       # This has happened to me (greater than 1 correlation estimate)
       if(any(alpha.new >= 1)){
         stop <- T
@@ -415,7 +428,10 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
       R.alpha.inv <- R.alpha.inv*phi.old/phi
     }else if(cor.match == 7){
       # USER SPECIFIED
-      alpha.new <- updateAlphaUser(Y, mu, phi, id, len, StdErr, Resid, p, BlockDiag, user.row, user.col, corr.list, included, includedlen, allobs, sqrtW)
+      alpha.new <- updateAlphaUser(Y, mu, phi, id, len, StdErr, Resid,
+                                   p, BlockDiag, user.row, user.col,
+                                   corr.list, included, includedlen,
+                                   allobs, sqrtW, useP)
       R.alpha.inv <- getAlphaInvUser(alpha.new, len, struct.vec, user.row, user.col, row.vec, col.vec)/phi
     }else if(cor.match == 1){
       # INDEPENDENT
@@ -424,8 +440,9 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     }
 
 
-    beta.list <- updateBeta(Y, X, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, W)
+    beta.list <- updateBeta(Y, X, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, W, included)
     beta <- beta.list$beta
+    
     phi.old <- phi
     if( max(abs((beta - beta.old)/(beta.old + .Machine$double.eps))) < tol ){converged <- T; stop <- T}
     if(count >= maxit){stop <- T}
@@ -453,7 +470,8 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
 
   eta <- as.vector(X %*% beta) + off
   if(sandwich){
-    sandvar.list <- getSandwich(Y, X, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, beta.list$hess, StdErr, dInvLinkdEta, BlockDiag, W)
+    sandvar.list <- getSandwich(Y, X, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, 
+                                beta.list$hess, StdErr, dInvLinkdEta, BlockDiag, W, included)
   }else{
     sandvar.list <- list()
     sandvar.list$sandvar <- "no sandwich"
@@ -478,7 +496,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   results$coefnames <- colnames(X)
   results$niter <- count
   results$converged <- converged
-  results$naiv.var <- solve(beta.list$hess)*phi.new  ## call model-based
+  results$naiv.var <- solve(beta.list$hess)  ## call model-based
   results$var <- sandvar.list$sandvar
   results$call <- call
   results$corr <- cor.vec[cor.match]
@@ -492,6 +510,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   results$terms <- modterms
   results$y <- Y
   results$biggest.R.alpha <- biggest.R.alpha/phi
+  results$formula <- formula
   class(results) <- "geem"
   return(results)
 }
@@ -502,17 +521,20 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
 
 
 ### Simple moment estimator of dispersion parameter
-updatePhi <- function(YY, mu, VarFun, p, StdErr, included, includedlen, sqrtW){
+updatePhi <- function(YY, mu, VarFun, p, StdErr, included, includedlen, sqrtW, useP){
   nn <- sum(includedlen)
+  
   resid <- diag(StdErr %*% included %*% sqrtW %*% Diagonal(x = YY - mu))
-  phi <- (1/(sum(included)-p))*crossprod(resid, resid)
-
+  
+  phi <- (1/(sum(included)- useP * p))*crossprod(resid, resid)
+  
   return(as.numeric(phi))
 }
 
 ### Method to update coefficients.  Goes to a maximum of 10 iterations, or when
 ### rough convergence has been obtained.
-updateBeta = function(YY, XX, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, W){
+updateBeta = function(YY, XX, beta, off, InvLinkDeriv, InvLink,
+                      VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, W, included){
   beta.new <- beta
   conv=F
   for(i in 1:10){
@@ -522,12 +544,16 @@ updateBeta = function(YY, XX, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.
     mu <- InvLink(eta)
     diag(StdErr) <- sqrt(1/VarFun(mu))
 
-    hess <- crossprod(  StdErr %*% dInvLinkdEta %*%XX, R.alpha.inv %*% W %*% StdErr %*%dInvLinkdEta %*% XX)
-    esteq <- crossprod(   StdErr %*%dInvLinkdEta %*%XX , R.alpha.inv %*% W %*% StdErr %*% as.matrix(YY - mu)) 
-
+    #hess <- crossprod( StdErr %*% dInvLinkdEta %*%XX, R.alpha.inv %*% W %*% StdErr %*%dInvLinkdEta %*% XX)
+    #esteq <- crossprod( StdErr %*%dInvLinkdEta %*%XX , R.alpha.inv %*% W %*% StdErr %*% as.matrix(YY - mu))
+    
+    hess <- crossprod( StdErr %*% dInvLinkdEta %*%XX, included %*% R.alpha.inv  %*% W %*% StdErr %*%dInvLinkdEta %*% XX)
+    esteq <- crossprod( StdErr %*%dInvLinkdEta %*%XX , included %*% R.alpha.inv %*% W %*% StdErr %*% as.matrix(YY - mu))
+    
+    
     update <- solve(hess, esteq)
-    if(max(abs(update)/beta.new) < 100*tol){break}
-
+    
+    
     beta.new <- beta.new + as.vector(update)
 
   }
@@ -535,7 +561,9 @@ updateBeta = function(YY, XX, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.
 }
 
 ### Calculate the sandiwch estimator as usual.
-getSandwich = function(YY, XX, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, hessMat, StdErr, dInvLinkdEta, BlockDiag, W){
+getSandwich = function(YY, XX, eta, id, R.alpha.inv, phi, InvLinkDeriv,
+                       InvLink, VarFun, hessMat, StdErr, dInvLinkdEta,
+                       BlockDiag, W, included){
 
   diag(dInvLinkdEta) <- InvLinkDeriv(eta)
   mu <- InvLink(eta)
@@ -543,7 +571,8 @@ getSandwich = function(YY, XX, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink,
   scoreDiag <- Diagonal(x= YY - mu)
   BlockDiag <- scoreDiag %*% BlockDiag %*% scoreDiag
 
-  numsand <- as.matrix(crossprod(  StdErr %*% dInvLinkdEta %*% XX, R.alpha.inv %*% W %*% StdErr %*% BlockDiag %*% StdErr %*% W %*% R.alpha.inv %*%  StdErr %*% dInvLinkdEta %*% XX))
+  #numsand <- as.matrix(crossprod(  StdErr %*% dInvLinkdEta %*% XX,  R.alpha.inv %*% W %*% StdErr %*% BlockDiag %*% StdErr %*% W %*% R.alpha.inv %*%  StdErr %*% dInvLinkdEta %*% XX))
+  numsand <- as.matrix(crossprod(  StdErr %*% dInvLinkdEta %*% XX, included %*% R.alpha.inv %*% W %*% StdErr %*% BlockDiag %*% StdErr %*% W %*% R.alpha.inv %*% included %*% StdErr %*% dInvLinkdEta %*% XX))
   
   sandvar <- t(solve(hessMat, numsand))
   sandvar <- t(solve(t(hessMat), sandvar))
